@@ -1,23 +1,25 @@
-environment : ModuleParent(parent = baseenv()) %type% {
-  # This is the type for the initial parent of a module.
-  # It knows of some functions:
-  parent.env(.Object) <- .Object@parent
-  makeDelayedAssignment("modules", "import", into = .Object)
-  makeDelayedAssignment("modules", "use", into = .Object)
-  makeDelayedAssignment("modules", "expose", into = .Object)
-  makeDelayedAssignment("modules", "export", into = .Object)
-  .Object
+ModuleParent <- function(parent = baseenv()) {
+
+  obj <- new.env(parent = parent)
+
+  makeDelayedAssignment("modules", "import", into = obj)
+  makeDelayedAssignment("modules", "use", into = obj)
+  makeDelayedAssignment("modules", "expose", into = obj)
+  makeDelayedAssignment("modules", "export", into = obj)
+  
+  obj
+  
 }
 
-environment : ModuleScope(parent ~ ModuleParent) %type% {
+ModuleScope <- function(parent = ModuleParent()) {
   # This is the type to wrap a module. It is the enclosing env of all funs in a
   # module
-  parent.env(.Object) <- .Object@parent
+  obj <- new.env(parent = parent)
   # Here are also the flags. Because of imports it might be hard to find the
   # original name-value for the exports. And to avoid multiple copies it is
-  # directly in the module env
-  assign(nameExports(), "^*", envir = .Object)
-  .Object
+  # directly in the module env. The value can be changed by 'expose'.
+  assign(nameExports(), "^*", envir = obj)
+  obj
 }
 
 ModuleConst <- function(expr, topEncl) {
@@ -35,27 +37,23 @@ ModuleConst <- function(expr, topEncl) {
     else exports
   }
 
-  wrapModfun <- function(module) {
-    # wrap all functions in a module with the class modfun.
-    mapInEnv(module, modfun, is.function)
-  }
-
   addModuleConst <- function(module) {
+    # This adds the moduleConst as an attribute to give each new module the
+    # possibility to create new instances.
     attr(module, "moduleConst") <- moduleConst
     module
   }
 
   new <- function() {
-
+    
     module <- ModuleScope(parent = ModuleParent(topEncl))
     module <- evalInModule(module, expr)
-    module <- wrapModfun(module)
     module <- retList("module", public = getExports(module), envir = module)
     addModuleConst(module)
 
   }
 
-  moduleConst <- stripSelf(retList("ModuleConst", c("new", "expr", "topEncl")))
+  moduleConst <- retList("ModuleConst", "new")
   moduleConst
 
 }
