@@ -2,7 +2,9 @@
 #'
 #' Use \code{module} to define self contained organizational units. Modules have
 #' their own search path. \code{import} can be used to import packages.
-#' \code{use} can be used to import other modules.
+#' \code{use} can be used to import other modules. Use \code{export} to define
+#' which objects to export. \code{expose} can be used to reuse function
+#' definitions from another module.
 #'
 #' @param expr an expression
 #' @param topEncl (environment) the root of the local search path. It is tried
@@ -11,13 +13,14 @@
 #' @param ... (character, or unquoted expression) names to import from package
 #'   or names to export from module. For exports a character of length 1 with a
 #'   leading "^" is interpreted as regular expression.
-#' @param where (environment) important for testing
+#' @param where (environment) typically the calling environment. Should only be
+#'   relevant for testing.
 #' @param module (character | module) a module as file or folder name or a list
 #'   representing a module.
 #' @param attach (logical) whether to attach the module to the search path
-#' @param x a module
-#' @param reInit (logical) whether to re-initialize module. This argument is
-#'   passed to \link{as.module}.
+#' @param reInit (logical) whether to re-initialize module. This is only
+#'   relevant if a module has \emph{state} which can be changed. This argument
+#'   is passed to \link{as.module}.
 #'
 #' @details
 #' \code{topEncl} is the environment where the search of the module ends.
@@ -29,50 +32,115 @@
 #' likely) means that the module is part of a package. In that case the module
 #' defines a sub unit within a package but has access to the packages namespace.
 #' This is relevant when you use the function module explicitly. When you define
-#' a nested module which the module will connect to the calling environment.
+#' a nested module the search path connects to the environment of the enclosing
+#' module.
 #'
-#' \code{import} and \code{use} are no replacements for \link{library} and
-#' \link{attach}. Both will work when called in the \code{.GlobalEnv} but should
-#' only be used for development and debugging of modules.
+#' \code{import} and \code{use} can replace \link{library} and \link{attach}.
+#' However they behave differently and are only designed to be used within
+#' modules. Both will work when called in the \code{.GlobalEnv} but should only
+#' be used for development and debugging of modules.
 #'
 #' \code{export} will never export a function with a leading "." in its name.
 #'
 #' \code{expose} is similar to \code{use} but instead of attaching a module it
-#' will copy all elements into the calling environment. This means that all
-#' functions will be re-exported; if not stated otherwise using \code{export}.
+#' will copy all elements into the calling environment. This means that
+#' \emph{exposed} functions can be (re-)exported.
 #'
 #' @examples
 #' \dontrun{
 #' vignette("modulesInR", "modules")
 #' }
 #'
+#' m <- module({
+#'   fun <- function(x) x
+#' })
+#'
+#' m$fun(1)
+#'
+#' m <- module({
+#'
+#'   import("stats", "median")
+#'   export("fun")
+#'
+#'   fun <- function(x) {
+#'     ## This is an identity function
+#'     ## x (ANY)
+#'     x
+#'   }
+#'
+#' })
+#'
+#' m$fun
+#' m
+#'
 #' @rdname module
 #' @export
 module <- function(expr = {}, topEncl = autoTopEncl(parent.frame())) {
 
-  ModuleConst(match.call()$expr, topEncl) %invoke% new()
+  moduleConstructor <- ModuleConst(match.call()$expr, topEncl)
+  moduleConstructor$new()
 
 }
 
 #' @export
-#' @rdname module
 print.module <- function(x, ...) {
-  for (i in seq_along(x)) {
-    cat(names(x)[i], ":\n", attr(x[[i]], "formals"), sep = "")
-    cat("\n\n")
+  
+  getFormals <- function(fun) {
+    formalsOfFun <- formals(fun)
+    formalsOfFun[sapply(formalsOfFun, is.character)] <-
+      lapply(formalsOfFun[sapply(formalsOfFun, is.character)], function(el){
+        paste0("\"", el, "\"")
+      })
+    args <- ifelse(
+      as.character(formalsOfFun) == "",
+      names(formalsOfFun),
+      paste(names(formalsOfFun), formalsOfFun, sep = " = ")
+    )
+    paste0("function(", paste(args, collapse = ", "), ")")
   }
+
+  getDoc <- function(fun) {
+    sourceOfFun <- stringr::str_trim(attr(fun, "srcref"))
+    sourceOfFun <- sourceOfFun[grep("^##", sourceOfFun)]
+    paste(sourceOfFun, collapse = "\n")
+  }
+
+  catFuns <- function(funs) {
+    for (i in seq_along(funs)) {
+      cat(names(funs)[i], ":\n", sep = "")
+      cat(getFormals(funs[[i]]), "\n", sep = "")
+      docString <- getDoc(funs[[i]])
+      if (length(docString) > 0) cat(docString, "\n", sep = "")
+      cat("\n", sep = "")
+    }
+  }
+  
+  catRemaining <- function(remaining) {
+    for (i in seq_along(remaining)) {
+      cat(paste0(names(remaining)[i], ":\n"))
+      cat(str(remaining[[i]]))
+      cat("\n")
+    }
+  }
+
+  ind <- vapply(x, is.function, logical(1))
+  catFuns(x[ind])
+  catRemaining(x[!ind])
+  
   invisible(x)
+  
 }
 
 #' @rdname module
 #' @export
-import <- function(from, ..., where = parent.frame()) {
+import <- function(from, ..., attach = TRUE, where = parent.frame()) {
 
   deparseImports <- function(mc) {
     args <- Map(deparse, mc)
     args[[1]] <- NULL
     args$from <- NULL
     args$where <- NULL
+    args$attach <- NULL
     args <- unlist(args)
     deleteQuotes(args)
   }
@@ -94,9 +162,10 @@ import <- function(from, ..., where = parent.frame()) {
 
   from <- deparseFrom(match.call())
   if (isNotInstalled(from)) stop("'package:", from, "' is not installed! Install first.")
+  if (!attach) where <- new.env()
   objectsToImport <- makeObjectsToImport(match.call(), from)
   addDependency(from, objectsToImport, where, makeDelayedAssignment, from)
-  invisible(NULL)
+  invisible(parent.env(where))
 
 }
 
@@ -131,7 +200,7 @@ use <- function(module, ..., attach = FALSE, reInit = TRUE, where = parent.frame
 expose <- function(module, ..., reInit = TRUE, where = parent.frame()) {
 
   mc <- match.call(expand.dots = TRUE)
-  mc[[1]] <- quote(use)
+  mc[[1]] <- quote(modules::use)
   module <- eval(mc, where)
 
   makeAssignment(module, names(module), where)
